import java.applet.Applet;
import java.awt.*;
import java.applet.*;
import java.math.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.Arrays;
import java.math.*;

/**This file tests the piecewise affine map
   from the plaid PET to the graph PET.**/

public class DebugEdgeTask implements Runnable {
    boolean HALT=true;

    public DebugEdgeTask() {
    }


    /**This generates the avoidance list.  We print the problems
       lexicographically, according to the pixellation list. 
       The pixellation list is associated to the plaid triple. 
       It is simply the list of two edges in the central square
       crossed by the plaid polygon.*/

 
    public void run() {
       int[][] list=getProblem();
       int count=0;
       System.out.println("public static int[] index(int k) {");
       System.out.print("int[][] a={");


	for(int j=0;j<100;++j) {
	for(int i=0;i<list.length;++i) {
	    int k=10*list[i][2]+list[i][3];
	    if(k==j) {
		++count;
		System.out.print("{"+list[i][0]+","+list[i][1]+","+list[i][2]+","+list[i][3]+","+direction(list[i])+"}");
	       if(count<462) System.out.print(",");
	    }
	}
	}
	System.out.println("};");
	System.out.println("return a[k];");
	System.out.println("}");
    }


    public void printBlank(int n) {
	for(int i=0;i<n;++i) System.out.print(" ");
    }


    public boolean normalMatching(boolean[][] m) {
	if((m[0][0]==true)&&(m[1][1]==true)) {
	    if((m[1][0]==false)||(m[0][1]==false)) return true;
	}

	if((m[0][1]==true)&&(m[1][0]==true)) {
	    if((m[0][0]==false)||(m[0][0]==false)) return true;
	}
	return(false);
    }

    /**This decides on the correct ordering.*/
    public int ordering(int k) {
	int[][] c=new int[2][2];
	int[][][] p=getPotential(k);
	int[] a=getPixellation(k);
	boolean[][] test=new boolean[2][2];
	for(int i=0;i<2;++i) {
	    for(int j=0;j<2;++j) {
		c[i][j]=overlap(a[i],p[j]);
	    }
	}
	int c0=c[0][0]*c[1][1];
	int c1=c[0][1]*c[1][0];
	if(c0>c1) return 0;
	if(c1>c0) return 1;
	return(-1);
    }

    public int[][] getProblem() {
	int count=0;
	int[][] list=new int[1000][4];
	for(int m=6;m<218;++m) {
	    int[][] a=getProblem(0,m);
	    if(a!=null) {
	    for(int i=0;i<a.length;++i) {
		list[count]=a[i];
		++count;
	    }}
	    a=getProblem(1,m);
	    if(a!=null) {
	    for(int i=0;i<a.length;++i) {
		list[count]=a[i];
		++count;
	    }}
	}
	return ListHelp.trim(list,count);
    }

    public int[][] getProblem(int choice,int m) {
	int count=0;
	int[][] list=new int[2][4];
	int[] a=getPixellation(m);
	int a0=a[choice];
	int[][][] p=getPotential(m);
	int[][] p0=p[choice];
	int pc=(choice+DataTripleIndex.ordering(m))%2;
	for(int i=0;i<p0.length;++i) {
	  for(int j=1;j<3;++j) {
	    boolean test=false;
	    if(p0[i][j]==a0) test=true;
	    if(test==false) {
		list[count][0]=m;
		list[count][1]=pc;
	        list[count][2]=p0[i][0];
	        list[count][3]=p0[i][j];
		++count;
	    }
	  }
	}
	if(count==0) return null;
	if(count==1) {
	    int[][] list2={list[0]};
	    return list2;
	}
	return list;
    }




    /**This counts the size of the intersection between the
       pixellation and the potential.*/


    public int overlap(int a,int[][] p) {
	int count=0;
        for(int i=0;i<p.length;++i) {
	   for(int j=1;j<p[i].length;++j) {
	   if(p[i][j]==a) ++count;
	   }
	}
	return(count);
    }



    /**Get correct crossings*/

    public int[] getPixellation(int i) {
            int[] c=DataTripleCodes.data(i);
	    int[] X={c[2],c[3]};
	    return X;
    }





    /**These get the potential ways that the given type can
       cross the edges of the square.  Some of them we want
       to rule out.*/

    public int[][][] getPotential(int j) {
	int[][][] N={getPotentialPlus(j),getPotentialMinus(j)};
	int t=DataTripleIndex.ordering(j);
	if(t==0) return N;
	int[][][] M={N[1],N[0]};
	return M;
    }

    public int[][] getPotentialPlus(int i) {
        int[][] t=DataTripleIndex.typePlus(i);
	int[][] u=new int[t.length][3];
	for(int j=0;j<t.length;++j) {
	   int k=Spider.convert(t[j][0]+1,t[j][1]+1);
	   int[] l=edgeCross(k);
	   u[j][0]=k;
	   u[j][1]=l[0];
	   u[j][2]=l[1];
	}
	return u;
    }


    public int[][] getPotentialMinus(int i) {
        int[][] t=DataTripleIndex.typeMinus(i);
	int[][] u=new int[t.length][3];
	for(int j=0;j<t.length;++j) {
	   int k=Spider.convert(t[j][0]+1,t[j][1]+1);
	   int[] l=edgeCross(k);
	   u[j][0]=k;
	   u[j][1]=l[0];
	   u[j][2]=l[1];
	}
	return u;
    }




    /**Here k denotes one of the 8 local arithmetic graph types.
       This routine returns the list of two walls which the type
       can potentially cross.  In terms of the absolute (i.e.
       untransformed) arithmetic graph, the indexing of the types
       is like this:

 
       2 4 7
       1   6
       0 3 5

       but then the edges are in a different position because of
       the affine transformation that is applied to the absolute
       picture.  The edges of the square are indexed as:


         2
       1   3
         0

       The crossing types are independent of the paramter*/


    public int[] edgeCross(int k) {
	if(k==-1) return null;
	int[][] t={{0,1},{0,1},{0,3},{1,2},{0,3},{1,2},{2,3},{2,3}};
	return t[k];
    }


    /**this decides if the edge involved in the avoidance problem
       is going against the grain of the plaid triple or with it.*/

    public int direction(int[] a) {
	int[] b=edgeCross(a[2]);
	int pix=b[0];
	if(a[3]==b[0]) pix=b[1];
	int[] t=DataTripleCodes.data(a[0]);
        if(pix==t[2]) return 0;
        if(pix==t[3]) return 1;
	return(-1);
    }



}

