import java.applet.Applet;
import java.awt.*;
import java.applet.*;
import java.math.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.Arrays;
import java.math.*;

/**This file tests the piecewise affine map
   from the plaid PET to the graph PET.**/

public class DebugPlaidToGraph implements Runnable {
    boolean HALT=true;

    public DebugPlaidToGraph() {
    }

    public void run() {
	mainTest();
    }




    public void specialPoint() {
	HALT=false;
	Parameter PP=new Parameter(0,0,"");
	int[] pp=PP.random();
	int p=pp[0];
	int q=pp[1];
	double A=1.0*p/q;
	double P=2*A/(1+A);
	int w=p+q;

	while(HALT==false) {
	    int i=(int)(w*w*Math.random());
	    int j=(int)(w*Math.random()); 
            if(AGLattice.goodPoint(p,q,i+.5,j+.5)==true) {
	        Complex p1=AGLattice.nearestGrid(p,q,i+.5,j+.5);
		Complex p2=Complex.plus(p1,new Complex(1,-P));
		if(p2.y>j) {
		    double[] d1=PlaidClassifyingMap.mapBasic(P,i+0.5,j+.5);
		    double[] d2=PlaidClassifyingMap.mapBasic(P,i+1.5,j+.5);
	            double[] d3={d2[0]-d1[0],d2[1]-d1[1],d2[2]-d1[2]};
		    double[] e1=PlaidToGraph.convert(d1);
		    double[] e2=PlaidToGraph.convert(d2);
		    double[] e3={e2[0]-e1[0],e2[1]-e1[1],e2[2]-e1[2]};

		    int reg1=PlaidToGraph.getRegion(d1);
		    int reg2=PlaidToGraph.getRegion(d2);


		   if(near(P,d3)==true) {
		    System.out.println(d3[0]+" "+d3[1]+" "+d3[2]+"    "+P);
		    System.out.println(reg1+" "+reg2);
		   }


		}
	    }
	}

    }

    public boolean bad1(double[] d1,double P) {
	if(d1[0]<1-2*P) return false;
	if(d1[0]>d1[1]) return false;
	return true;
    }


    public boolean bad2(double[] d1,double P) {
	if((d1[0]>1-2*P)&&(d1[0]>d1[1]+2-P)) return true;
	return false;
    }





    public boolean near(double P,double[] d) {
	if(Math.abs(d[0]-(2*P))>.00001) return false;
	if(Math.abs(d[1]-(2*P-2))>.00001) return false;
	return true;
    }







    public void particle() {
	Parameter PP=new Parameter(0,0,"");
	int[] pp=PP.random();
	int p=pp[0];
	int q=pp[1];
	double A=1.0*p/q;
	double P=2*A/(1+A);
	int w=p+q;
	HALT=false;

	while(HALT==false) {
	    int i0=(int)(Math.random()*w*w);
	    int j0=(int)(Math.random()*w);
	    int a=MathRational.tune(p,q,w-1);
	    int i1=i0+a*w;
	    int j1=j0+1;


	double[] U0=PlaidClassifyingMap.mapBasic(P,i0+.5,j0+.5);
	double[] U1=PlaidClassifyingMap.mapBasic(P,i1+.5,j1+.5);
	Vector4 V0=new Vector4(U0[0],U0[1],U0[2],U0[3]);
	Vector4 V1=new Vector4(U1[0],U1[1],U1[2],U1[3]);
	Vector4 W0=PlaidToGraph.convert(V0);
	Vector4 W1=PlaidToGraph.convert(V1);
	Vector4 V2=Vector4.minus(V1,V0);
	Vector4 W2=Vector4.minus(W1,W0);
	System.out.println("---");
	V2.print();
	W2.print();
	System.out.println(2*A);
	System.out.println(2*P);
	}
    }



    public void printout(double[] d) {
	System.out.println(d[0]+" "+d[1]+" "+d[2]+" "+d[3]);
    }



    public void mainTest() {
	HALT=false;
	int p=1;
	int q=2;
	while(HALT==false) {
	    testConvert(p,q);
	    ++p;
	    if(p==q) {
		++q;
		p=1;
	    }
	}
    }


    public void testConvert(int p,int q) {


	if(MathRational.GCD(p,q)!=1) return;
	if((p*q)%2==1) return;
	System.out.print("plaid2graph test "+p+"/"+q+"  ");
	int w=p+q;

	Complex TARGET=new Complex(w*w/2.0,w/2.0);


	double A=1.0*p/q;
	double P=2*A/(1+A);

	for(int i=-w*w;i<2*w*w;++i) {
	    for(int j=-w;j<2*w;++j) {
              if(AGLattice.goodPoint(p,q,i+.5,j+.5)==true) {
                Complex u=AGLattice.nearestGrid(p,q,i+.5,j+.5);
	        double[] d1=AGMap.reduceLattice(A,u.x,u.x,u.x);
		double[] U=PlaidClassifyingMap.mapBasic(P,i+.5,j+.5);
		double[] d2=PlaidToGraph.convert(U);
		d2=AGMap.reduceLattice(A,d2[0],d2[1],d2[2]);

		boolean test=almostEqual(p,q,d1,d2);
		if(test==false) {
		    System.out.println("");
		    System.out.println(d1[0]+" "+d1[1]+" "+d1[2]);
		    System.out.println(d2[0]+" "+d2[1]+" "+d2[2]);
                    throw new ProofException("plaid to graph fail");
		}
	      }
	    }
	}
	System.out.println("done");
    }




    public static boolean almostEqual(int p,int q,double[] a,double[] b) {
	double A=1.0*p/q;
	for(int i=0;i<3;++i) {
	    if(Math.abs(a[i]-b[i])>.00000001) return(false);
	}
	return(true);
    }

    public static boolean almostEqual1(int p,int q,double[] a,double[] b) {
	double A=1.0*p/q;
	for(int i=0;i<1;++i) {
	    if(Math.abs(a[i]-b[i])>.00000001) return(false);
	}
	return(true);
    }

}



