import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

/**This checks the main pixellation theorem in the paper.
There are 462 instances where we have a plaid triple and
a graph edge assignment which could potentially violate
the pixellation phenomenon:  The edge from the graph should
cross the square side the same way that the plaid model does.
For all but 40 of these cases we check that pixellation holds.
For the remaining 40 we see by inspection (or by running the
ProofCoherence file) that the adjacent integer square is
graph empty.  This is part of the claim of the paper.

We run two kinds of tests. Some of the cases are eliminated
by the spider test, which checks things in the graph PET.
The others are eliminated by the plaid test and the
outside test, which test things in the plaid PET.  The
test generates the list of failures.  We don't need the
list, but we want to see that it is the same list as the
list of problem indices that we have stored in the
DataAvoidance file.*/


public class ProofEdgeCrossing {

    public static void main() {
	System.out.println("");
	System.out.println("public static int[] problem(int k) {");
	System.out.print("int[][] a={");
	int count=0;
	for(int i=0;i<462;++i) {
	    boolean deb=false;
	    if(i==92) deb=true;
	    int[] I=DataEdgeCrossing.index(i);
	    int test=spiderTest(I);
	    if(test==-1) test=plaidTest(I);
	    if(test==-1) test=outsideTest(I);
	    if(test==-1) {
                 System.out.print("{"+I[0]+","+I[1]+","+I[2]+","+I[3]+","+I[4]+"}");
		 if(count<45) System.out.print(",");
		 if(count==45) System.out.println("};");
		 ++count;
	    }
	}
	System.out.println("return a[k];");
	System.out.println("}");
	System.out.println(count);
    }



    public static int plaidTest(int[] I) {
	int choice=-1;
	if((I[2]==0)&&(I[3]==0)) choice=0;
	if((I[2]==0)&&(I[3]==1)) choice=1;
	if((I[2]==1)&&(I[3]==1)) choice=7;
	if((I[2]==3)&&(I[3]==1)) choice=2;
	if((I[2]==3)&&(I[3]==2)) choice=4;
	if((I[2]==4)&&(I[3]==0)) choice=2;
	if((I[2]==4)&&(I[3]==3)) choice=4;
	if((I[2]==7)&&(I[3]==2)) choice=5;
	if((I[2]==6)&&(I[3]==3)) choice=6;
	if((I[2]==7)&&(I[3]==3)) choice=3;
	if(choice==-1) return -1;
	PolytopeLong P=AllPolytopesLong.plaidTriple(I[0]);
	PolytopeLong Q=AllPolytopesLong.plaidBarrier(choice);
	boolean inside=true;
	if(choice==0) inside=false;
	if(choice==5) inside=false;
	if((inside==true)&&(Q.contains(P)==true)) return 1;
	if((inside==false)&&(Q.isDisjoint(P)==true)) return 1;
	return -1;
    }


    public static int outsideTest(int[] I) {
	PolytopeLong P=AllPolytopesLong.plaidTriple(I[0]);
	for(int k=8;k<12;++k) {
	    PolytopeLong Q=AllPolytopesLong.plaidBarrier(k);
	    if(Q.contains(P)==true) return 1;
	}
	return -1;
    }


    /**This gets the relevant polytopes to be tested*/

    public static PolytopeLong[] getPolys(int[] I) {	
        int[][] a=DataTripleIndex.indexPlus(I[0]);
	if(I[1]==1) a=DataTripleIndex.indexMinus(I[0]);
	int[][] b=DataTripleIndex.typePlus(I[0]);
	if(I[1]==1) b=DataTripleIndex.typeMinus(I[0]);
	int index=-1;
	for(int i=0;i<b.length;++i) {
	   int m=Spider.convert(b[i][0]+1,b[i][1]+1);
	   if(m==I[2]) index=i;
	}
	PolytopeLong P1=AllPolytopesLong.graph(a[index]);
	PolytopeLong P2=AllPolytopesLong.graphTriple(I[0]);
	PolytopeLong[] P={P1,P2};
	return P;
    }


    /**These routines all have to do with deciding whether a polytope 
       lies to one side of a given hyperplane.  The first argument in
       all these routines specifies the list of hyperplanes.*/

    /**Here is the main routine.  A "container" is a finite set of
       halfspaces, as given by a list of integer vectors.  In all
       cases there are just 1 or 2 halfspaces in a container. The
       left argument is a finite list of containers, either 1,2,or 3.
       The right argument is a list of polytopes, either 1 or 2.
       The function returns true if at least one polytope is
       contained in at least one container.*/


    public static int spiderTest(int[] I) {
	PolytopeLong[] P=getPolys(I);
        long[][] W=getBarriers(I[2],I[3]);
	int test=-1;
	if(W==null) return(-1);
	return nonpositive(W,P);
    }


    public static int nonpositive(long[][] W,PolytopeLong[] P) {
       for(int j=0;j<P.length;++j) {
	  if(nonpositive(W,P[j])==true) return 0;
       }
	return -1;
    }

    /**all have to be nonpositive for a true return*/
    public static boolean nonpositive(long[][] W,PolytopeLong P) {
	for(int i=0;i<W.length;++i) {
	    if(nonpositive(W[i],P)==false) return false;
	}
	return true;
    }

    /**Geometrically this asks whether the polytope lies in the
       positive half space determined by the affine functional V.*/

    public static boolean nonpositive(long[] V,PolytopeLong P) {
	for(int i=0;i<P.count;++i) {
	    long[] W={P.V[i].x[0],P.V[i].x[1],P.V[i].x[2],P.V[i].x[3],1};
	    if(evaluate(V,W)<0) return false;
	}
	return true;
    }


    public static long evaluate(long[] V,long[] W) {
	long s=0;
	for(int i=0;i<5;++i) s=s+V[i]*W[i];
	return(s);
    }

    /**Here are the affine functionals*/

    public static long[][] getBarriers(int k,int k1) {
	if((k==1)&&(k1==1)) return barriers11();
	if((k==2)&&(k1==3)) return barriers23();
	if((k==3)&&(k1==2)) return barriers32();
	if((k==4)&&(k1==0)) return barriers40();
	if((k==5)&&(k1==1)) return barriers51();
	if((k==6)&&(k1==3)) return barriers63();
	return null;
    }

    public static long[][] barriers11() {
	long[][] W={{1,0,0,-1,0},{-1,0,0,0,60}};
	return(W);
    }

    public static long[][] barriers23() {
	long[][] W={{-1,0,0,+1,0},{1,0,0,0,0}};
	return(W);
    }

    public static long[][] barriers32() {
	long[][] W={{0,1,0,-1,0},{0,-1,0,0,60}};
	return(W);
    }

    public static long[][] barriers40() {
	long[][] W={{0,1,0,-2,0},{0,-1,0,1,60}};
	return(W);
    }

    public static long[][] barriers51() {
	long[][] W={{1,0,0,1,-60},{-1,0,0,0,60}};
	return(W);
    }

    public static long[][] barriers63() {
	long[][] W={{-1,0,0,-1,60},{1,0,0,0,0}};
	return(W);
    }
}



